@@ -2,6 +2,7 @@
  * This file is part of KDevelop
  *
  * Copyright 2016 Carlos Nihelton <carlosnsoliveira@gmail.com>
+ * Copyright 2018 Friedrich W. H. Kossebau <kossebau@kde.org>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -27,6 +28,8 @@
 #include "config/clangtidypreferences.h"
 #include "config/clangtidyprojectconfigpage.h"
 #include "job.h"
+#include "problemmodel.h"
+#include "utils.h"
 // KDevPlatform
 #include <interfaces/icore.h>
 #include <interfaces/context.h>
@@ -42,8 +45,8 @@
 #include <project/interfaces/ibuildsystemmanager.h>
 #include <project/projectconfigpage.h>
 #include <project/projectmodel.h>
-#include <shell/problemmodel.h>
 #include <shell/problemmodelset.h>
+#include <util/jobstatus.h>
 // KF
 #include <KActionCollection>
 #include <KLocalizedString>
@@ -54,6 +57,7 @@
 #include <QMessageBox>
 #include <QMimeType>
 #include <QMimeDatabase>
+#include <QThread>
 
 using namespace KDevelop;
 
@@ -75,7 +79,7 @@ bool isSupportedMimeType(const QMimeType& mimeType)
 
 Plugin::Plugin(QObject* parent, const QVariantList& /*unused*/)
     : IPlugin(QStringLiteral("kdevclangtidy"), parent)
-    , m_model(new KDevelop::ProblemModel(parent))
+    , m_model(new ProblemModel(this, this))
 {
     setXMLFile(QStringLiteral("kdevclangtidy.rc"));
 
@@ -84,14 +88,10 @@ Plugin::Plugin(QObject* parent, const QVariantList& /*unused*/)
     connect(m_checkFileAction, &QAction::triggered, this, &Plugin::runClangTidyFile);
     actionCollection()->addAction(QStringLiteral("clangtidy_file"), m_checkFileAction);
 
-    /*     TODO: Uncomment this only when discover a safe way to run clang-tidy on
-    the whole project.
-    //     QAction* act_check_all_files;
-    //     act_check_all_files = actionCollection()->addAction ( "clangtidy_all",
-    this, SLOT ( runClangTidyAll() ) );
-    //     act_check_all_files->setText(i18n("Analyze Current Project with Clang-Tidy)"));
-    //     act_check_all_files->setIcon(QIcon::fromTheme(QStringLiteral("dialog-ok")));
-    */
+    m_checkProjectAction = new QAction(QIcon::fromTheme(QStringLiteral("dialog-ok")),
+                                       i18n("Analyze Current Project with Clang-Tidy"), this);
+    connect(m_checkProjectAction, &QAction::triggered, this, &Plugin::runClangTidyAll);
+    actionCollection()->addAction(QStringLiteral("clangtidy_project"), m_checkProjectAction);
 
     ProblemModelSet* pms = core()->languageController()->problemModelSet();
     pms->addModel(Strings::modelId(), i18n("Clang-Tidy"), m_model.data());
@@ -123,9 +123,11 @@ void Plugin::unload()
     pms->removeModel(Strings::modelId());
 }
 
+
 void Plugin::updateActions()
 {
     m_checkFileAction->setEnabled(false);
+    m_checkProjectAction->setEnabled(false);
 
     if (isRunning()) {
         return;
@@ -136,16 +138,16 @@ void Plugin::updateActions()
         return;
     }
 
-    if (!isSupportedMimeType(activeDocument->mimeType())) {
-        return;
-    }
-
     auto currentProject = core()->projectController()->findProjectForUrl(activeDocument->url());
     if (!currentProject) {
         return;
     }
 
-    m_checkFileAction->setEnabled(true);
+    if (isSupportedMimeType(activeDocument->mimeType())) {
+        m_checkFileAction->setEnabled(true);
+    }
+
+    m_checkProjectAction->setEnabled(true);
 }
 
 
@@ -169,6 +171,13 @@ void Plugin::runClangTidy(const QUrl& url, bool allFiles)
         return;
     }
 
+    m_model->reset(project, url, allFiles);
+
+    const auto buildDir = project->buildSystemManager()->buildDirectory(project->projectItem());
+
+    QString error;
+    const auto filePaths = Utils::filesFromCompilationDatabase(buildDir, url, allFiles, error);
+
     ClangTidyProjectSettings projectSettings;
     projectSettings.setSharedConfig(project->projectConfiguration());
     projectSettings.load();
@@ -180,14 +189,9 @@ void Plugin::runClangTidy(const QUrl& url, bool allFiles)
     auto clangTidyPath = KDevelop::Path(ClangTidySettings::clangtidyPath()).toLocalFile();
     params.executablePath = clangTidyPath;
 
-    if (allFiles) {
-        params.filePath = project->path().toUrl().toLocalFile();
-    } else {
-        params.filePath = url.toLocalFile();
-    }
-    if (const auto buildSystem = project->buildSystemManager()) {
-        params.buildDir = buildSystem->buildDirectory(project->projectItem()).toLocalFile();
-    }
+    params.filePaths = filePaths;
+    params.buildDir = buildDir.toLocalFile();
+
     params.additionalParameters = projectSettings.additionalParameters();
 
     const auto enabledChecks = projectSettings.enabledChecks();
@@ -202,13 +206,29 @@ void Plugin::runClangTidy(const QUrl& url, bool allFiles)
     params.headerFilter = projectSettings.headerFilter();
     params.checkSystemHeaders = projectSettings.checkSystemHeaders();
 
+    params.parallelJobCount =
+        ClangTidySettings::parallelJobsEnabled() ?
+            (ClangTidySettings::parallelJobsAutoCount() ?
+                QThread::idealThreadCount() :
+                ClangTidySettings::parallelJobsFixedCount()) :
+        1;
+
     auto job = new ClangTidy::Job(params, this);
+    connect(job, &Job::problemsDetected, m_model.data(), &ProblemModel::addProblems);
     connect(job, &KJob::finished, this, &Plugin::result);
+    core()->uiController()->registerStatus(new KDevelop::JobStatus(job, i18n("Clang-Tidy")));
     core()->runController()->registerJob(job);
 
     m_runningJob = job;
 
     updateActions();
+    showModel();
+}
+
+void Plugin::showModel()
+{
+    ProblemModelSet* pms = core()->languageController()->problemModelSet();
+    pms->showModel(Strings::modelId());
 }
 
 bool Plugin::isRunning() const
@@ -235,12 +255,11 @@ void Plugin::result(KJob* job)
         return;
     }
 
+    m_model->finishAddProblems();
+
     if (aj->status() == KDevelop::OutputExecuteJob::JobStatus::JobSucceeded ||
         aj->status() == KDevelop::OutputExecuteJob::JobStatus::JobCanceled) {
-        m_model->setProblems(aj->problems());
-
-        ProblemModelSet* pms = core()->languageController()->problemModelSet();
-        pms->showModel(Strings::modelId());
+        showModel();
     } else {
         core()->uiController()->findToolView(i18ndc("kdevstandardoutputview", "@title:window", "Test"), nullptr,
                                              KDevelop::IUiController::FindFlags::Raise);
@@ -260,6 +279,9 @@ ContextMenuExtension Plugin::contextMenuExtension(Context* context, QWidget* par
             connect(action, &QAction::triggered, this, &Plugin::runClangTidyFile);
             extension.addAction(KDevelop::ContextMenuExtension::AnalyzeFileGroup, action);
         }
+        auto action = new QAction(QIcon::fromTheme(QStringLiteral("dialog-ok")), i18n("Clang-Tidy"), parent);
+        connect(action, &QAction::triggered, this, &Plugin::runClangTidyAll);
+        extension.addAction(KDevelop::ContextMenuExtension::AnalyzeProjectGroup, action);
     }
 
     if (context->hasType(KDevelop::Context::ProjectItemContext) && !isRunning()) {
@@ -269,14 +291,18 @@ ContextMenuExtension Plugin::contextMenuExtension(Context* context, QWidget* par
             return extension;
         }
 
-        auto item = items.first();
-
-        if (item->type() != KDevelop::ProjectBaseItem::File) {
+        const auto item = items.first();
+        const auto itemType = item->type();
+        if ((itemType != KDevelop::ProjectBaseItem::File) &&
+            (itemType != KDevelop::ProjectBaseItem::Folder) &&
+            (itemType != KDevelop::ProjectBaseItem::BuildFolder)) {
             return extension;
         }
-        const QMimeType mimetype = QMimeDatabase().mimeTypeForUrl(item->path().toUrl());
-        if (!isSupportedMimeType(mimetype)) {
-            return extension;
+        if (itemType == KDevelop::ProjectBaseItem::File) {
+            const QMimeType mimetype = QMimeDatabase().mimeTypeForUrl(item->path().toUrl());
+            if (!isSupportedMimeType(mimetype)) {
+                return extension;
+            }
         }
 
         auto action = new QAction(QIcon::fromTheme(QStringLiteral("dialog-ok")), i18n("Clang-Tidy"), parent);
